@@ -113,14 +113,14 @@ int csys_fs::read(const std::string &path, int *ptr_val) {
 
 	int fd = ::open(p.c_str(), O_RDONLY);
 	if (fd < 0) {
-		thd_log_warn("sysfs read failed %s\n", path.c_str());
+		thd_log_warn("sysfs open failed %s\n", path.c_str());
 		return -errno;
 	}
 	ret = ::read(fd, str, sizeof(str));
 	if (ret > 0)
 		*ptr_val = atoi(str);
 	else
-		thd_log_warn("sysfs read failed %s\n", path.c_str());
+		thd_log_info("sysfs read failed %s\n", path.c_str());
 	close(fd);
 
 	return ret;
@@ -180,6 +180,17 @@ bool csys_fs::exists(const std::string &path) {
 	return (bool) (stat((base_path + path).c_str(), &s) == 0);
 }
 
+int csys_fs::create() {
+
+	int fd = ::open(base_path.c_str(), O_CREAT | O_WRONLY | O_TRUNC, S_IRWXU);
+	if (fd < 0) {
+		thd_log_warn("sysfs open failed %s\n", base_path.c_str());
+		return -errno;
+	}
+	close(fd);
+	return 0;
+}
+
 bool csys_fs::exists() {
 	return csys_fs::exists("");
 }
